/*
MAX TO FUSION EXPORTER 
Created by Tim Little
Contact me with bug reports or feedback
timhlittle@gmail.com
timhlittle.prosite.com

Bake selected scene objects and export an FBX file which can be loaded into Fusion. Offers greater control and flexibility than the standard FBX exporter.

Feature List:
+ Automatically bake all object transformations to ensure a pixel perfect match between Max and Fusion renders
+ Preserve complex camera animation (Camera Target, Camera FOV, Camera Roll, etc.)
+ Export a Child object without including the geometry of its Parent objects. 
+ Bake Particle Flow systems and export particles as animated pieces of geometry (Scale, Rotation, and Position)
+ Shift all animation to begin at frame zero (Useful when a Fusion Comp begins at frame zero while the Max scene has longer heads and tails)
+ Automatically interpret and bake Out-Of-Range key methods (such as looping) for position, rotation, and scale of any object


Installation:
Drag and drop the downloaded MZP file on to an open window of 3ds Max. An installer dialog will open automatically.

VERSION 1.1

*/

--===========================================================================================--

-- Close UI if it already exists
try destroyDialog ExportSettingsRollout catch()


--============================--
--// LOAD THE FBX PLUGIN //--
--============================--
pluginManager.loadClass FBXEXP


--============================--
--// DEFINE GLOBAL VARIABLES //--
--============================--
global temporaryFreeCameras = #()
global sceneTargetCameras = #()

global temporaryBakedObjects = #()
global sceneObjects = #()

global objlist = #()
global tempParentList = #()

global oldUnitScale = units.SystemScale
global oldUnitType = units.SystemType

global iconPath = (GetDir #scripts + "\\LittleScripts\\icons\\" + "MaxToFusion_Banner.bmp")
if not doesFileExist (iconPath) then iconPath = ""


--============================--
--// DEFINE FUNCTIONS //--
--============================--

function exportFBXFile fileName filePath allOrSelectArg= 
(

	fbxClass = undefined
	
	disableSceneRedraw()
	
--// Store User Input //--
	
	if allOrSelectArg == 1 then allOrSelectArg = true
	if allOrSelectArg == 2 then allOrSelectArg = false
	
	
	fbxFileName = (filePath + fileName)
	print fbxFileName
	
	if doesfileexist fbxFileName then
	(
		overwrite = true
		--fbxFileName = fbxFileName + "_1"
	)
	else
	(
		overwrite = false
	)
	
--// create directory to output fbx to //--
	--makeDir defaultFilePath all:true --Currently unnecessary to create directory

--// Export FBX //--

	tableOfExportClasses = exporterPlugin.classes
	for i=1 to tableOfExportClasses.count do
	(
		item = (tableOfExportClasses[i] as string)
		
		if (matchPattern item pattern:"FBXEXP" ignoreCase:true ) then fbxClass=i
	)
	
	if fbxClass == undefined then
	(
		messageBox "FBX Plugin was not found. Could not export"
		return ("FAILED")
	)
	
	FBXExporterSetParam "Animation" true
	FbxExporterSetParam "Cameras" true
	FBXExporterSetParam "BakeAnimation" true
	FbxExporterSetParam "ShowWarnings" true
	FBXExporterSetParam "FileVersion" "FBX200611" --this is version 2010
	exportResult = exportFile (fbxFileName) #noPrompt selectedOnly:allOrSelectArg using: tableOfExportClasses[fbxClass] -- FBX is the 9th export type
	
	print "export Result is ..."
	print exportResult
	
	fileCheck = doesfileexist fbxFileName
	
	if (exportResult == true and fileCheck == true) then
	(
			if (queryBox (fileName + " was exported sucessfully. \nWould you like to reveal the new file in Windows Explorer?") title: "SUCCESS") then
			(
				ShellLaunch "explorer.exe" filePath
			)
	)
	else(messageBox "File export failed for some reason :( ")

) -- end of exportFBXFile function



--// Recursively select the parents of a given object //--
function selectAllParents obj =
(
		if obj.parent != undefined and obj.parent != "undefined" then
		(
			append objlist obj.parent
			append tempParentList obj.parent
			selectMore obj.parent
			selectAllParents obj.parent
		)
)


--// Bake Particle Systems //--
function processParticleSystems particleNodeListArg particleTypeArg timeRange customTimeStart customTimeEnd particleQuantity leaveBakedObjects integrationStep =
(

	print "Processing particle systems..."
	
	-- Check to see if time range is set to Active Time Range
	if timeRange == 1 then 
	(
		rangeStart = animationRange.start
		rangeEnd = animationRange.end
	)
	
	-- Check to see if time range is set to Custom Time Range
	if timeRange == 2 then 
	(
		rangeStart = customTimeStart
		rangeEnd = customTimeEnd
	)		
	
	
	animateVisibility = true
	
	pflowArray = particleNodeListArg

	global bakedPointHelpersArray = #()

	with redraw on for pf in pflowArray do
	(	
		
		pf = getNodeByName pf
		
		--Make sure pf is a valid node
		try(print pf.name)
		catch(return #())
		
		-- Syncronize the integration step of the particle flow system
		pf.Integration_for_Viewport = pf.Integration_for_Render 
			
		-- Set the number of steps per frame
		--integrationStep = 1

		-- Either use the viewport particles or all particles
		orginalViewportQuantity = pf.Quantity_Viewport
		if particleQuantity == 2 then pf.Quantity_Viewport = pf.Quantity_Render
		if particleQuantity == 1 then pf.Quantity_Viewport = pf.Quantity_Viewport
			
		pointHelperArray = #()
		
		pfGroupDummy = dummy name: (pf.name + "_Group")
		hide pfGroupDummy
		
		-- Store Default Key Tangents
		user_inTangentType = BezierDefaultParams.inTangentType
		user_outTangentType = BezierDefaultParams.outTangentType
		
		-- Set Default Key Tangents to Linear
		BezierDefaultParams.inTangentType = #linear
		BezierDefaultParams.outTangentType = #linear

		for f = rangeStart to rangeEnd by integrationStep do 
		(

			cnt = pf.numParticles()
			
			sliderTime = f
			canceled = false
			
			--pointHelper = Point prefix:"PHelper_001"
			
			for n = 1 to cnt where not canceled do 
			(
				local i = pf.getParticleID n

				
				if pf.getParticleAgeByID i > 0f then 
				(
					
					-- Get the particle transform.
					local tm = pf.getParticleTMByID i
					
					-- Get the particle scale
					local pScale = pf.getParticleScaleByID i
					
					-- Get the particle shape
					local pShape = pf.getParticleShapeByID  i
					
					-- check to see if the particle partner to match the point exists yet
					if tm != undefined then
					(
						if particleTypeArg == 1 then 
						(
							pointHelper = plane width:20 length:20 lengthsegs:1 widthsegs:1 prefix: (pf.name + "_Card_001")
						)
						if particleTypeArg == 2 then 
						(
							pointHelper = sphere radius:10 segs:16 prefix: (pf.name + "_Sphere_001")
						)
						if particleTypeArg == 3 then 
						(
							pointHelper = box length:10 width:10 height:10 prefix: (pf.name + "_Cube_001")
						)
						if particleTypeArg == 4 then 
						(
							pointHelper = convertToMesh (box length:1 width:1 height:1 prefix: (pf.name + "_Mesh_001"))
							
							try (setMesh pointHelper pShape)
							catch()
						)
						
						hide pointHelper
						append pointHelperArray pointHelper
						
						-- Bake the particle's transform
						try
						(
							
							animate on pointHelperArray[i].transform = tm
								
							animate on pointHelperArray[i].scale = pf.getParticleScaleXYZ(i)
							
							setUserProp pointHelperArray[i] "UsedByParticleSystem" true
						)
						catch(print "couldn't add User Prop")
						
						
-- 						--Bake the particle's visibility. Feature not fully implemented yet
-- 						if leaveBakedObjects == true then
-- 						(
-- 							pointHelper.visibility = bezier_float()
-- 							pointHelper.visibility.controller.value = 0
-- 							at time (f - 1t) animate on 
-- 								pointHelper.visibility.controller.value = 0
-- 							animate on
-- 								pointHelper.visibility.controller.value = 1
-- 						)
						
					)
				)
			)
			-- Animate the particles off visibility, if needed. Feature not fully implemented yet
-- 			if leaveBakedObjects == true then
-- 			(
-- 				for i = 1 to pointHelperArray.count where pointHelperArray[i] != undefined and pointHelperArray[i].visibility and pf.getParticleIndex i == 0 do 
-- 				(
-- 					at time (f - 1t) animate on
-- 						pointHelperArray[i].visibility.controller.value = 1
-- 					animate on
-- 						pointHelperArray[i].visibility.controller.value = 0
-- 				)
-- 			)
			
		) -- end of stepping through frames
		
		-- Reset Default Key Tangents
		BezierDefaultParams.inTangentType = user_inTangentType
		BezierDefaultParams.outTangentType = user_outTangentType

		--// Kill any unused helper objects that were created //--
		for pHelper in pointHelperArray do
		(
			
			deleteKey pHelper.position.controller.keys 1
			deleteKey pHelper.rotation.controller.keys 1
			deleteKey pHelper.scale.controller.keys 1
			
			isUsedByParticleSystem = (getUserProp pHelper "UsedByParticleSystem")
			
			if isUsedByParticleSystem == undefined then 
			(
				delete pHelper
			)
			else
			(
				pHelper.parent = pfGroupDummy
				append bakedPointHelpersArray pHelper
			)
		)
		deselect bakedPointHelpersArray
		pf.Quantity_Viewport = orginalViewportQuantity
		return bakedPointHelpersArray
		
	)-- end of For PF in pflowArray loop
	
) -- end of function Process Particles



--// Creates a point helper for every selected object which is used to hold animation until it can be baked back onto the original object //--
function bakeAnimation moveAnimationtoZero forceUpdate selectedOrAll includeNonSelectedParents enableParticles particleType timeRange customTimeStart customTimeEnd processedParticlesArray obj_TransformControllerArray enableFramePaddingChbx frameHeads frameTails framePaddingObjects =
(
	
	if forceUpdate then format "Enforcing full viewport update. This might be slow but will ensure proper updating of all controllers\n"
	
	objlist = undefined
	
	-- Store Default Key Tangents
	user_inTangentType = BezierDefaultParams.inTangentType
	user_outTangentType = BezierDefaultParams.outTangentType
	
	-- Set Default Key Tangents to Linear
	BezierDefaultParams.inTangentType = #linear
	BezierDefaultParams.outTangentType = #linear
	
	if selectedOrAll == 2 then -- (1 means only process user selection, 2 means process the entire scene
	(
		objlist = objects as array
		select objects
	)
	else
	(
		-- selectionArray is a global variable defined in the rollout function
		objlist = selection as array
	)
	
	hasNoParent = undefined
	
	--// Grab all parent objects not included in original selection //--
	for obj in objlist do
	(
		if obj.parent != undefined and obj.parent != "undefined" then selectAllParents obj
	)

	
	for bake_obj in objlist do 	-- for every select object do
	(
		try(
			
			-- Check to see if time range is set to Active Time Range
			if timeRange == 1 then 
			(
				bakeTimeStart = animationRange.start
				bakeTimeEnd = animationRange.end
			)
			
			-- Check to see if ti me range is set to Custom Time Range
			if timeRange == 2 then 
			(
				bakeTimeStart = customTimeStart
				bakeTimeEnd = customTimeEnd
			)	
			
			objClass = ClassOf bake_obj
			isBakedParticle = getUserProp bake_obj "UsedByParticleSystem"
			
-- 			if (isBakedParticle == "true" or isBakedParticle == true) then format "Baking Camera Properties of %\n" bake_obj.name
-- 			else format "Collapsing transformation of object %\n" bake_obj.name
			
			-- Check to see if the object has any animation on its transforms. If no animaton Don't worry about baking it!
			if bake_obj.transform.isAnimated == false and objClass != VRayPhysicalCamera then
			(
				-- do not bake this object
			)
			
			if bake_obj.transform.isAnimated == false and objClass == VRayPhysicalCamera then
			(
				print "Adding keys to a non-animated vray camera"
				addNewKey bake_obj.transform.controller animationRange.start
				addNewKey bake_obj.transform.controller animationRange.End
				
				if bake_obj.transform.isAnimated then print "IS NOW ANIMATED!!!" else print "didn't work"
			)
			
			if bake_obj.transform.isAnimated == true then
			(
				local p = undefined
				local old_prs_ctrl = copy bake_obj.transform.controller		-- store old controller for catch()
				append obj_TransformControllerArray old_prs_ctrl
				
				if not forceUpdate then disableSceneRedraw();
				-- disableSceneRedraw is problematic as not all scripted controllers are updated
				try 
				(
					-- If obj is a camera then duplicate it
					if (objClass == targetcamera or objClass == freecamera) then duplicateCameraObject = copy bake_obj
					if (objClass == VRayPhysicalCamera) then maxCam = FreeCamera name:(bake_obj.name + "_Converted") type:#free
					
					p = Point()			-- create temp point object
					
					if enableFramePaddingChbx  == true then
					(
						--Cameras Only
						if framePaddingObjects == 1 then 
						(
							if (objClass == freecamera or objClass == VRayPhysicalCamera or objClass == targetcamera) then
							(
								--print "Using Frame Padding for Cameras Only"
								bakeTimeStart = bakeTimeStart - frameHeads 
								bakeTimeEnd = bakeTimeEnd + frameTails 
							)
						)
						if framePaddingObjects == 2 then 
						(
							--print "Using Frame Padding for All Objects"
							bakeTimeStart = bakeTimeStart - frameHeads 
							bakeTimeEnd = bakeTimeEnd + frameTails 
						)
					)
					
					-- copy global transform of source object into temp object
					for i = bakeTimeStart to bakeTimeEnd do 
					(
						if forceUpdate then sliderTime = i	-- set slider time to force a global update of all animation data
						at time i (
							with animate on p.transform = bake_obj.transform
						)
					)
					
					-- kill old transform controller and assign new, clean one
					if (objClass != VRayPhysicalCamera) then
					(
						bake_obj.transform.controller = transform_script()	
						bake_obj.transform.controller = prs()
					)
					
					
					if (objClass == targetcamera or objClass == freecamera) then
					(
						bake_obj.type = #free
						deletekeys bake_obj
						enableORTs bake_obj false
						
						format "Baking Camera Properties of %\n" bake_obj.name
					)
					if (objClass == VRayPhysicalCamera) then
					(
						deselect bake_obj
						vrayCam = bake_obj
						selectMore maxCam
					)
					
					
					-- copy temp object animation back into source object
					for i = bakeTimeStart to bakeTimeEnd do 
					(
						at time i 
						(
							with animate on
							(
								bake_obj.transform = p.transform
								
								newObjClass = ClassOf bake_obj
								
								if (newObjClass == freecamera) then
								(
									
									bake_obj.fov = duplicateCameraObject.fov
									bake_obj.nearclip = duplicateCameraObject.nearclip
									bake_obj.farclip = duplicateCameraObject.farclip
									
									--Other, currently unused parameters
									--bake_obj.targetDistance = duplicateCameraObject.targetDistance
									--bake_obj.useTargetDistance = duplicateCameraObject.useTargetDistance
									--bake_obj.focalDepth = duplicateCameraObject.focalDepth
									--bake_obj.nearrange = duplicateCameraObject.nearrange
									--bake_obj.farrange = duplicateCameraObject.farrange
									--bake_obj.mpassEnabled = duplicateCameraObject.mpassEnabled
									--bake_obj.mpassRenderPerPass = duplicateCameraObject.mpassRenderPerPass
								)
								
								
								if (objClass == VRayPhysicalCamera) then
								(
									maxCam.transform = p.transform
									maxCam.mpassEffect.focalDepth = vrayCam.focus_distance
									maxCam.fov = vrayCam.fov
									
								)
								
							)
						)
					) --end of for loop to copy back animation
					
					if moveAnimationtoZero == true then
					(
						difference = (0 - animationRange.start)
						moveKeys bake_obj difference
						
						try
						(
							difference = (0 - animationRange.start)
							moveKeys vrayCam difference
							moveKeys maxCam difference
						)
						catch()
						
					)
					
					try
					(
						if includeNonSelectedParents then deselect tempParentList
					)
					catch(print "failed to deselect non selected parents")
						
					--try delete p
					--catch()
					
					--try delete duplicateCameraObject
					--catch()
					
					try deselect vrayCam
					catch()
					
					if not forceUpdate then enableSceneRedraw()
				) catch (
					format "Fatal error - exiting\n"
					if p!=undefined then delete p
					bake_obj.transform.controller = old_prs_ctrl
					if not forceUpdate then enableSceneRedraw()
					
					return "FAILED"
				) -- catch 
			) -- end of baking for loop if the object is animated
		)
		catch()
	) -- for bake_obj in...
	
	-- Reset Default Key Tangents
	BezierDefaultParams.inTangentType = user_inTangentType
	BezierDefaultParams.outTangentType = user_outTangentType

	try (selectMore processedParticlesArray)
	catch(print "No baked particles to select")
	
	
	if objlist.count == 0 then (
		format "No object selected\n"
	)

) --end of bakeAnimation function


function particleSystem_filt obj =
(
	if obj.category == #Particle_Systems then return true
	else return false
	
)

function NOparticleSystem_filt obj =
(
	if obj.category != #Particle_Systems then return true
	else return false
	
)


function startUI = 
(
	--// DEFINE VARIABLES //--

	global currentProjectRoot = pathConfig.getcurrentProjectFolder()
	global fbxOutputDirectory = "\\exports\\MaxToFusion\\"
	global defaultExportDirectory = (currentProjectRoot + fbxOutputDirectory)
	global defaultFileName = maxFileName  
	fileNameCount = defaultFileName.count as integer
	defaultFileName = substring defaultFileName 1 (fileNameCount-4) --cuts off the .max part of the name

	if defaultFileName == "" then defaultFileName = "Untitled"
	
	defaultFileName = defaultFileName + "_M2F.fbx"
	
	global bakeResult = undefined
	global userExportPath = undefined
	global particleNodeList = #()
	global particleList = #()
	global selectionArray = undefined	
	global allParticles = #()
	global selectedParticles =#()
	global obj_TransformControllerArray = #()
	global instructionText = "+ Use the 'Choose Objects' Button to select the \n objects you wish to be exported.\n \n+ To export non-selected parent objects as a \n simple null instead of its complete geo, select \n'Export Non-Selected Parents as Null Objects' \n \n+ Right-click the export button to disable scene \ncleanup and leave baked objects after export \n \n+ To export your selected ParticleFlow systems \n check the 'Enable Particles' button. Baking all the \n particles will be slow and it is recommended you do \n not export more than 500 particles at a time. \n \n+ Vertex animation on particle shapes is not \ncurrently supported but XYZ scaling will work. \n \n+ Vray Camera support is currently in beta. \n Horizontal & Vertical Offset parameters will not work. \n \n+ Frame Padding Option allows extra frames of\n animation to be baked beyond the timerange for \naccurate motion blur on the first and last frames.\nParticles currently not supported"
	global cleanUpScene	= true
	
	--// DEFINE Rollouts //--
	rollout ExportSettingsRollout "Max To Fusion v1.1" width:300 height:561
	(
		bitmap Banner_Btmp "Bitmap" pos:[0,0] width:300 height:37 fileName:(iconPath)
	
		GroupBox grp1 "Selection" pos:[4,46] width:290 height:94
		radiobuttons selectedOrAllRBtns "" pos:[108,71] width:222 height:16 labels:#("Selection Only", "Entire Scene") columns:2 across:2 visible:false default:1
		button selectByNameBtn "Choose Objects" pos:[12,70] width:120 height:30 across:2
		label selectionListLabel "0 Objects Chosen" bold:true  style_sunkenedge:false pos:[155,82] --pos:[14,95]
		listbox hiddenObjectSelectionListBox "SECRET" items:#() visible:false
		checkbox includeNonSelectedParentsBtn "Export Non-Selected Parents as Null Objects" pos:[13,117] width:235 height:15 checked:true toolTip:"If checked, any parents in a heirarchy which you do not explicitly select will appear in fusion as an invisible null object (with animation intact)"
		
		--------------------------------------------------------
		
		GroupBox grp2 "Particle Options" pos:[4,143] width:290 height:122
		checkbox enableParticlesChbx "Enable Particle Export" pos:[13,160] height:15 checked:false
		radiobuttons particleTypeRBtn "Bake Particles As" pos:[198,180] width:209 height:30 labels:#("Cards", "Spheres", "Cubes","Particle Shape") columns:1 enabled:false

		listBox particleListBox items:#("0 PFlow Systems Chosen") height:2 width: 172 readonly:true pos:[15,183] enabled:false
		radiobuttons particleQuantity labels:#("Viewport %","Render %") pos:[13,220] enabled:false
		spinner particleStepSpnnr "Integration Step" range:[0.125,1,1] scale:0.125 width:80 enabled:false align:#left
		
		--------------------------------------------------------
		GroupBox grp3 "Frame Padding Options" pos:[4,267] width:290 height:96
		checkbox enableFramePaddingChbx "Enable Frame Padding" pos:[13,285] height:15 checked:false toolTip:"If checked, the script will include object animation beyond the chosen Time Range"
		
		spinner frameHeadsSpnnr "Heads:" pos:[25,311] width:70 height:16 enabled:false range:[0,9999,5] type:#integer across:2
		spinner frameTailsSpnnr "Tails:" pos:[125,311] width:70 height:16 enabled:false range:[0,9999,5] type:#integer
		
		radiobuttons framePaddingObjects "" pos:[15,335] width:262 height:16 labels:#("Cameras Only", "All Objects") default:1 columns:2 enabled:false
		
		--------------------------------------------------------
 
		GroupBox grp4 "Export Options" pos:[4,370] width:290 height:180
		
		radiobuttons timeRangeRBtn "" pos:[15,389] width:262 height:16 labels:#("Active Time Range", "Custom Range") default:1 columns:2
		spinner customTimeStartSpnnr "Start:" pos:[25,411] width:70 height:16 enabled:false range:[0,9999,0] type:#integer across:2
		spinner customTimeEndSpnnr "End:" pos:[125,411] width:70 height:16 enabled:false range:[0,9999,100] type:#integer
				
		checkbox startAnimationFrameZeroCbx "Move Animation to begin at frame Zero" pos:[13,435] height:15 checked:true
		checkbox forceRedrawCbx "Force Scene Redraw When Baking Animation" pos:[13,450] width:260 height:15 checked:false toolTip:"Redraw Max scene each frame of the timeline to ensure accurate transform data for time-dependent animation. WARNING: This can be very slow when exporting many objects"		
				
		edittext userFileNameEdtx "File Name:" pos:[16,470] width:204 height:17 labelOnTop:true readOnly:true
		button incrementFileNameBtn "+" pos:[224,487] width:12 height:18
		button browseUserExportPathBtn "Browse" pos:[238,487] width:51 height:18
				
		button executeBtn "Export To Fusion" pos:[15,515] width:250 height:26
		checkbox leaveBakedObjectsChbox "Leave Baked Objects" checked:false visible:false -- hidden attribute holder
		button showInstructionsBtn "?" pos:[268,515] height:26
		
		---------------------------------------------------------
		
		edittext instructionsEdtx "Instructions:" text:instructionText pos:[10,45] height:420 width:276 labelOnTop:true readonly:true visible:false
		
		button hideInstructionsBtn "<<" pos:[268,515] height:26 width:20 visible:false
		label LittleScriptsLabel "For more tools visit:" pos:[15,520] visible:false
		hyperLink LittleScriptsLink "http://timhlittle.prosite.com" address:"http://timhlittle.prosite.com/100549/tools" visible:false pos:[120,520] color:[59, 149, 202] hoverColor:blue visitedColor:[59, 149, 202]

		
		--// EVENT HANDLERS //--
		
		on executeBtn rightclick do
		(
			if executeBtn.text == "Export To Fusion --LEAVE BAKED OBJECTS--" then
			(
				executeBtn.text = "Export To Fusion"
				leaveBakedObjectsChbox.checked = false
				cleanUpScene = true
				return()
			)
			
			if executeBtn.text == "Export To Fusion" then
			(
				executeBtn.text = "Export To Fusion --LEAVE BAKED OBJECTS--"
				leaveBakedObjectsChbox.checked = true
				cleanUpScene = false
				return()
			)
		)
		
		
		on showInstructionsBtn pressed do
		(
			
			hideInstructionsBtn.visible = true
			instructionsEdtx.visible = true
			LittleScriptsLink.visible = true
			LittleScriptsLabel.visible = true
			
			showInstructionsBtn.visible = false
			
			grp1.visible = false
			selectByNameBtn.visible = false
			selectionListLabel.visible = false
			includeNonSelectedParentsBtn.visible = false
			grp2.visible = false
			enableParticlesChbx.visible = false
			particleTypeRBtn.visible = false
			particleQuantity.visible = false
			particleListBox.visible = false
			particleStepSpnnr.visible = false
			grp3.visible = false
			grp4.visible = false
			timeRangeRBtn.visible = false
			customTimeStartSpnnr.visible = false
			customTimeEndSpnnr.visible = false
			startAnimationFrameZeroCbx.visible = false
			forceRedrawCbx.visible = false
			userFileNameEdtx.visible = false
			incrementFileNameBtn.visible = false
			browseUserExportPathBtn.visible = false
			executeBtn.visible = false
			
		)
		
		on hideInstructionsBtn pressed do
		(
			hideInstructionsBtn.visible = false
			instructionsEdtx.visible = false
			LittleScriptsLink.visible = false
			LittleScriptsLabel.visible = false
			
			showInstructionsBtn.visible = true
			
			grp1.visible = true
			selectByNameBtn.visible = true
			selectionListLabel.visible = true
			includeNonSelectedParentsBtn.visible = true
			grp2.visible = true
			enableParticlesChbx.visible = true
			particleTypeRBtn.visible = true
			particleQuantity.visible = true
			particleListBox.visible = true
			particleStepSpnnr.visible = true
			grp3.visible = true
			grp4.visible = true
			timeRangeRBtn.visible = true
			customTimeStartSpnnr.visible = true
			customTimeEndSpnnr.visible = true
			startAnimationFrameZeroCbx.visible = true
			forceRedrawCbx.visible = true
			userFileNameEdtx.visible = true
			incrementFileNameBtn.visible = true
			browseUserExportPathBtn.visible = true
			executeBtn.visible = true
		)

		
		
		on incrementFileNameBtn pressed do
		(
			fileName = userFileNameEdtx.text
			
			if fileName != "" and fileName != undefined then
			(
				tempFileName = substring fileName 1 (fileName.count-4)
				
				lastcharacter = substring tempFileName tempFileName.count tempFileName.count
				if lastcharacter == "t" then lastcharacter = undefined -- this catches an error. t for some reason resolves as 0 instead of undefined
				
				
				try(lastDigit = lastcharacter as integer)
				catch(lastDigit = undefined)
				
				-- If the last character is a number
				if (lastDigit != undefined) then 
				(
					-- If the number is less than 9 just add onto it
					if lastDigit != 9 then 
					(
						lastcharacter = (lastDigit + 1) as string
						newtext = replace tempFileName (tempFileName.count) 1 lastcharacter
						userFileNameEdtx.text = newtext + ".fbx"
					)
						
					-- If the number is 9 then we need to increment the number before it also
					if lastDigit == 9 then
					(
						--print lastDigit
						
						secondLastCharacter = substring tempFileName (tempFileName.count-1) (tempFileName.count-1)
						if secondLastCharacter == "t" then secondLastCharacter = undefined
						
						try(secondLastDigit = secondLastCharacter as integer)
						catch(secondLastDigit = undefined)
						
						if secondLastDigit != undefined then
						(
							secondLastDigit = secondLastDigit + 1
							lastDigit = 0
							
							lastTwoNumbers = (secondLastDigit + lastDigit) as string
							--print lastTwoNumbers
							
							newtext = replace tempFileName (tempFileName.count-1) 2 lastTwoNumbers
							userFileNameEdtx.text = newtext + ".fbx"
						)
					)
				)
				
				
				else if lastDigit == undefined then
				(
					userFileNameEdtx.text = tempFileName + "_01" + ".fbx"
				)
			)
			
		)
		
		on selectedOrAllRBtns changed state do
		(
			allParticles = #()
			
			if state == 1 then 
			(
				selectByNameBtn.enabled = true 
				includeNonSelectedParentsBtn.enabled = true
				selectionListLabel.enabled = true
			
				particleListBox.items = selectedParticles
				if particleListBox.items[1] == undefined then particleListBox.items = #("0 PFlow Systems Chosen")
			)
			if state == 2 then
			(
				selectByNameBtn.enabled = false	
				includeNonSelectedParentsBtn.enabled = false
				selectionListLabel.enabled = false
				
				for obj in Geometry do
				(
					if obj.category == #Particle_Systems then
					(
						try 
						(
							obj.numParticles()
							
							append allParticles obj.name
						)
						catch
						(
							-- this will ignore dumb particle systems like sprays and blizzards
						)
					)
					particleListBox.items = allParticles
				)
			)
		)
		
		on selectByNameBtn pressed do
		(
			selectedParticles =#()
			
			selectionArray = selectByName title:"Select the objects to Export" showHidden:true --filter:NOparticleSystem_filt
			
			
			if selectionArray != undefined then
			(
				for obj in selectionArray where (obj.category == #Particle_Systems) do 
				(
					try 
					(
						obj.numParticles()
						
						append selectedParticles obj.name as string
					)
					catch
					(
						-- this will ignore dumb particle systems like sprays and blizzards
					)
					
				)
				particleListBox.items = selectedParticles
				selectionListLabel.text = (selectionArray.count as string + " Objects Chosen")
				
				if particleListBox.items[1] == undefined then particleListBox.items = #("0 PFlow Systems Chosen")
			)
			
		)
		
		on enableParticlesChbx changed state do
		(
			if state == true then
			(
				particleTypeRBtn.enabled = true
				particleQuantity.enabled = true
				particleListBox.enabled = true
				particleStepSpnnr.enabled = true
				
				forceRedrawCbx.checked = false
				forceRedrawCbx.enabled = false
				
			)
			if state == false then 
			(
				particleTypeRBtn.enabled = false
				particleQuantity.enabled = false
				particleListBox.enabled = false
				particleStepSpnnr.enabled = false
				forceRedrawCbx.enabled = true
			)
		)
		
		on timeRangeRBtn changed state do
		(
			if state == 2 then 
			(
				customTimeStartSpnnr.enabled = true
				customTimeEndSpnnr.enabled = true
			)
			if state == 1 then
			(
				customTimeStartSpnnr.enabled = false
				customTimeEndSpnnr.enabled = false	
			)
		)
		
		on browseUserExportPathBtn pressed do
		(
			try
			(
				if userFileNameEdtx.text != undefined and userFileNameEdtx.text != "" then 
				(
					defaultFileName = userFileNameEdtx.text
				)
				
				userExportPath = getSaveFileName caption: "Export FBX To" filename: defaultFileName types:"FBX(*.fbx)" \
				historyCategory: "MaxToFusionExport"
				
				if	userExportPath != undefined then
				(
					
					userEnteredName = filenameFromPath userExportPath
					if (matchPattern userEnteredName pattern: "*.fbx" ignoreCase:true) then userFileNameEdtx.text = userEnteredName
					else userFileNameEdtx.text = (userEnteredName + ".fbx")
					
					userExportPath = getFilenamePath userExportPath
				)
			)
			catch(print"Cancel")
		)
		
		
		on enableFramePaddingChbx changed state do 
		(
			if state == true then 
			(
				frameHeadsSpnnr.enabled = true
				frameTailsSpnnr.enabled = true
				framePaddingObjects.enabled = true
			)
			if state == false then
			(
				frameHeadsSpnnr.enabled = false
				frameTailsSpnnr.enabled = false
				framePaddingObjects.enabled = false
			)
		)
		
		--// MAIN EXECUTION //--
		on executeBtn pressed do
		(
			global userFileName = userFileNameEdtx.text
			global userPath = userExportPath
			
			goAhead = false
			
			--// Check to see if anything is selected //--
			if (selectionArray == undefined or selectionArray[1] == undefined) then
			(	
				
				if selectedOrAllRBtns.state == 1 then
				(
					print "nothing selected >:O"
					messageBox "Nothing is selected. You gotta give me something to export!" title:"ERROR"
					return()
				)
				-- dont worry about selections becuase user is exporting entire scene
				else(goAhead = true)
				
				
			)
			else(goAhead = true)
			
			if goAhead == true then 
			(
				print "beginning export..."
				
				
			--// Check to see if the user entered a custom file path //--

			if userFileNameEdtx.text == undefined or userFileNameEdtx.text == "" then
			(
				
				messageBox "ERROR: No export path selected"
				return()
			)
			else print "using custom export path"
				
			--// Check to see which export method the user chose //--
			clearSelection()
				
			if selectedOrAllRBtns.state == 1 then
			(
				print selectionArray
				select selectionArray
			)
			else
			(
				select objects			
			)
				
			--// Check to see if the file name already exists in the output directory //--
				if doesfileexist (userPath + userFileName) then
				(
					print "duplicate file detected..."
					yesOverwrite = queryBox "WARNING: An FBX file of the same name already exists in the output folder. Exporting will overwrite that file. \nAre you sure you want to continue?" title: "CONFIRM OVERWRITE"
					
					--Overwrite the existing file
					if yesOverwrite then
					(
						print "overwriting existing file"
						
						holdMaxFile()
						
						--// Process Particle Systems //--
						if enableParticlesChbx.checked then
						(
							if particleListBox.items[1] != "0 Pflow Systems Chosen" then
							(
								processedParticlesArray = processParticleSystems particleListBox.items particleTypeRBtn.state timeRangeRBtn.state \
														customTimeStartSpnnr.value customTimeEndSpnnr.value particleQuantity.state leaveBakedObjectsChbox.checked particleStepSpnnr.value
							)
							else 
							(
								print "You enabled particles but didn't select any, what up with that?"
							)
						)
						--// Manually bake object animation //--
						bakeResult = bakeAnimation startAnimationFrameZeroCbx.checked forceRedrawCbx.checked selectedOrAllRBtns.state includeNonSelectedParentsBtn.checked \
													enableParticlesChbx.checked particleTypeRBtn.state timeRangeRBtn.state customTimeStartSpnnr.value \
													customTimeEndSpnnr.value processedParticlesArray obj_TransformControllerArray enableFramePaddingChbx.checked frameHeadsSpnnr.value \
													frameTailsSpnnr.value framePaddingObjects.state

						
						--hiddenObjectSelectionListBox.items = selectionArray
						
						if bakeResult == "FAILED" then
						(
							messageBox "Something went wrong while baking animation. Sorry :("
							return()
						)
						
						--// Include Particles //--

						exportFBXFile userFileName userPath selectedOrAllRBtns.state
			
						print "the final file name was...."
						print userFileName
					
					)
					-- Or Abort the export
					else 
					(
						print "aborted the export"
						return()
					)
				) --end of if does file exists
				
				-- If no duplicate file exists export awaaay!
				if not doesfileexist (userPath + userFileName) then
				(
					holdMaxFile()
					
					--// Process Particle Systems //--
						if enableParticlesChbx.checked then
						(
							print particleListBox.items[1]
							
							if particleListBox.items[1] as string != "0 Pflow Systems Chosen" then
							(
								processedParticlesArray = processParticleSystems particleListBox.items particleTypeRBtn.state timeRangeRBtn.state \
														customTimeStartSpnnr.value customTimeEndSpnnr.value particleQuantity.state leaveBakedObjectsChbox.checked particleStepSpnnr.value
							)
							else 
							(
								print "You enabled particles but didn't select any, what up with that?"
							)
						)
						--// Manually bake object animation //--
						bakeResult = bakeAnimation startAnimationFrameZeroCbx.checked forceRedrawCbx.checked selectedOrAllRBtns.state includeNonSelectedParentsBtn.checked \
													enableParticlesChbx.checked particleTypeRBtn.state timeRangeRBtn.state customTimeStartSpnnr.value \
													customTimeEndSpnnr.value processedParticlesArray obj_TransformControllerArray enableFramePaddingChbx.checked frameHeadsSpnnr.value \
													frameTailsSpnnr.value framePaddingObjects.state
					
						--hiddenObjectSelectionListBox.items = selectionArray
						
					if bakeResult == "FAILED" then
					(
						messageBox "Something went wrong while baking animation. Sorry :("
						return()
					)

					exportFBXFile userFileName userPath selectedOrAllRBtns.state
		
					print "the final file name was...."
					print userFileName
				)
			) --end of if something selected condition
			
			
			if goAhead == true then 
			(
				
				units.SystemScale = oldUnitScale
				units.SystemType = oldUnitType
				
				selectionArray = #()
				selectionListLabel.text = "0 0bjects Selected"
				particleListBox.items = #("0 PFlow Systems Chosen")
				
				enableSceneRedraw()
				
				if cleanUpScene == true then
				(
					fetchMaxFile quiet:true -- reset max back the way it was before the export baked everything
					try select selectionArray
					catch()
				)
				else
				(
					-- leave all the baked objects around
				)
				
			)
			
		)-- end ON execute button
		
	)--end of Rollout definition
	
	try(destroyDialog ExportSettingsRollout)
	catch()
	
	MaxToFusionDialog =	createDialog ExportSettingsRollout  \
						lockWidth:true LockHeight:true style:#(#style_titlebar, #style_sysmenu) 


) -- end of StartUI function definition

/*
================================================================================
======================	BEGIN SCRIPT	EXECUTION		============================
================================================================================
*/

print "beginning MaxFBXtoFusion Script ..."

startUI()

